/*
 * nomysql
 *
 * [mysql]
 * NoMysqlFile=visitors.txt
 *
 * Tracks last 10 players to enter the zone and the time (?last)
 * Writes player info to a text file for later examination (data/visitors.txt)
 *
 * 14/01/04 - Started by Smong. Based off aliasdb.c
 *
 * 15/01/04 - Time stamp taken from log_file.c. Client stats
 *  code from playercmd.c (?info). Now logs like this:
 *
 * month day hours:minute name:client/enc:ip:macid
 *
 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "asss.h"

#define suffix(n) (((n) == 1) ? "" : "s")

typedef struct
{
	char name[24];
	int time;
} PlayerLastLogin;

local void appendInfo(Player *p);
local void add_player(Player *p);
local void remove_all_players(void);
local void Clast(const char *params, Player *p, const Target *target);
local void PlayerAction(Player *p, int action, Arena *arena);

local LinkedList lastset;

local Imodman *mm;
local Iconfig *cfg;
local Icmdman *cmd;
local Ichat   *chat;
local Inet    *net;

local char default_file_name[] = { "visitors.txt" };
local char *type_names[] =
{
	"unknown", "fake", "vie", "ctm", "chatnet"
};

//Code from log_file.c
local void append_time(FILE *f)
{
	time_t t1;
	char t3[128];

	if (!f) return;

	time(&t1);
	strftime(t3, sizeof(t3), "%b %d %H:%M", localtime(&t1));
	fputs(t3, f);
}

local void appendInfo(Player *p)
{
	FILE *f;
	const char *fname;
	const char *type;
	char fullpath[48];
	struct net_client_stats s;
	
	fname = cfg->GetStr(GLOBAL, "mysql", "NoMysqlFile");
	if (!fname) fname = default_file_name;

	sprintf(fullpath, "data/%s", fname);
	f = fopen(fullpath, "a");
	if (!f) return;

	//Log time
	append_time(f);

	//Get IP + enc
	net->GetClientStats(p, &s);
	
	//Get type
	type = p->type >= 0 && p->type < (sizeof(type_names)/sizeof(type_names[0])) ?
		type_names[p->type] : "really_unknown";
		
	//Write to file
	fprintf(f, " %s:%s/%s:%s:%d\n", p->name, type, s.encname, s.ipaddr, p->macid);

	//Clean up
	fclose(f);
}

local void add_player(Player *p)
{
	Link *link;
	PlayerLastLogin *pll;
	
	//Update an entry
	for (link = LLGetHead(&lastset); link; link = link->next)
	{
		pll = link->data;
	
		//Does the player already exist?
		if (strcmp(pll->name, p->name) == 0)
		{
			//Then re-add and adjust time
			LLRemove(&lastset, pll);
			pll->time = current_ticks() / 100;
			LLAdd(&lastset, pll);
			return;
		}
	}
	
	//Maxed out? Recycle mem
	if (LLCount(&lastset) == 10)
	{
		pll = LLRemoveFirst(&lastset);
	}
	else
	{
		//Create new
		pll = amalloc(sizeof(PlayerLastLogin));
	}
	
	//Set values and add
	strcpy(pll->name, p->name);
	pll->time = current_ticks() / 100;
	LLAdd(&lastset, pll);
}

local void remove_all_players()
{
	Link *link;
	
	//Update an entry
	for (link = LLGetHead(&lastset); link; link = link->next)
	{
		afree(link->data);
	}
}

local helptext_t last_help =
	"Targets: none\n"
	"Args: none\n"
	"Tells you the last 10 people to log in.\n";

local void Clast(const char *params, Player *p, const Target *target)
{
	Link *link;
	PlayerLastLogin *pll;
	const char *name;
	int days, hours, mins, secs;
	int now = current_ticks() / 100;
	
	if (target->type != T_ARENA)
		return;
	
	for (link = LLGetHead(&lastset); link; link = link->next)
	{
		pll = link->data;
		secs = now - pll->time;
		name = pll->name;
	
		mins = secs / 60;
		secs %= 60;
		hours = mins / 60;
		mins %= 60;
		days = hours / 24;
		hours %= 24;
		
		if (days == 0)
		{
			if (hours == 0)
			{
				if (mins == 0)
				{
					chat->SendMessage(p, "%-20.20s  %d second%s ago",
						name, secs, suffix(secs));
				}
				else
				{
					chat->SendMessage(p, "%-20.20s  %d minute%s ago",
						name, mins, suffix(mins));
				}
			}
			else
			{
				chat->SendMessage(p, "%-20.20s  %d hour%s ago",
					name, hours, suffix(hours));
			}
		}
		else
		{
			chat->SendMessage(p, "%-20.20s  %d day%s ago",
				name, days, suffix(days));
		}
	}
}

local void PlayerAction(Player *p, int action, Arena *arena)
{
	if (action == PA_CONNECT)
	{
		add_player(p);
		appendInfo(p);
	}
}

EXPORT int MM_nomysql(int action, Imodman *mm_, Arena *arena)
{
	if (action == MM_LOAD)
	{
		mm = mm_;
		cfg  = mm->GetInterface(I_CONFIG, ALLARENAS);
		cmd  = mm->GetInterface(I_CMDMAN, ALLARENAS);
		chat = mm->GetInterface(I_CHAT,   ALLARENAS);
		net  = mm->GetInterface(I_NET,    ALLARENAS);
		if (!cfg || !cmd || !chat || !net)
			return MM_FAIL;

		LLInit(&lastset);
		mm->RegCallback(CB_PLAYERACTION, PlayerAction, ALLARENAS);
		cmd->AddCommand("last", Clast, last_help);

		return MM_OK;
	}
	else if (action == MM_UNLOAD)
	{
		cmd->RemoveCommand("last", Clast);
		mm->UnregCallback(CB_PLAYERACTION, PlayerAction, ALLARENAS);
		//LLFree(&lastset); //Hrm, this doesn't free the data
		remove_all_players();
		LLEmpty(&lastset);
		
		mm->ReleaseInterface(cfg);
		mm->ReleaseInterface(cmd);
		mm->ReleaseInterface(chat);
		mm->ReleaseInterface(net);
		return MM_OK;
	}

	return MM_FAIL;
}
